@@ -25,18 +25,29 @@ Tool for generating a sample configuration file. See
 
 import collections
 import copy
+import importlib.metadata
+import json
 import logging
 import operator
 import sys
-import textwrap
 
-import json
-import pkg_resources
-import six
+try:
+    # For oslo.config[rst-generator]
+    from docutils import core as docutils_core
+    from docutils.parsers.rst import nodes as docutils_nodes
+    from docutils.parsers.rst import roles as docutils_roles
+    import rst2txt
+    from sphinx import roles as sphinx_roles
+except ImportError:
+    import textwrap
+
+    rst2txt = None
+
+
 import yaml
 
-
 from oslo_config import cfg
+from oslo_i18n import _message
 import stevedore.named  # noqa
 
 LOG = logging.getLogger(__name__)
@@ -49,6 +60,7 @@ _generator_opts = [
     cfg.IntOpt(
         'wrap-width',
         default=70,
+        min=0,
         help='The maximum length of help lines.'),
     cfg.MultiStrOpt(
         'namespace',
@@ -108,18 +120,15 @@ def _format_defaults(opt):
             default_str = str(opt.sample_default)
         elif opt.default is None:
             default_str = '<None>'
-        elif (isinstance(opt, (cfg.StrOpt, cfg.IPOpt,
-                               cfg.HostnameOpt, cfg.HostAddressOpt,
-                               cfg.URIOpt))):
-            default_str = opt.default
+        elif isinstance(opt, (cfg.StrOpt, cfg.IntOpt, cfg.FloatOpt, cfg.IPOpt,
+                              cfg.PortOpt, cfg.HostnameOpt, cfg.HostAddressOpt,
+                              cfg.URIOpt, cfg.Opt)):
+            default_str = str(opt.default)
         elif isinstance(opt, cfg.BoolOpt):
             default_str = str(opt.default).lower()
-        elif isinstance(opt, (cfg.IntOpt, cfg.FloatOpt,
-                              cfg.PortOpt)):
-            default_str = str(opt.default)
         elif isinstance(opt, (cfg.ListOpt, cfg._ConfigFileOpt,
                               cfg._ConfigDirOpt)):
-            default_str = ','.join(six.text_type(d) for d in opt.default)
+            default_str = ','.join(str(d) for d in opt.default)
         elif isinstance(opt, cfg.DictOpt):
             sorted_items = sorted(opt.default.items(),
                                   key=operator.itemgetter(0))
@@ -131,8 +140,8 @@ def _format_defaults(opt):
 
     results = []
     for default_str in defaults:
-        if not isinstance(default_str, six.text_type):
-            default_str = six.text_type(default_str)
+        if not isinstance(default_str, str):
+            default_str = str(default_str)
         if default_str.strip() != default_str:
             default_str = '"%s"' % default_str
         results.append(default_str)
@@ -161,25 +170,54 @@ def _format_type_name(opt_type):
     return 'unknown value'
 
 
-class _OptFormatter(object):
+class _OptFormatter:
 
     """Format configuration option descriptions to a file."""
 
-    def __init__(self, output_file=None, wrap_width=70):
+    def __init__(self, conf, output_file=None):
         """Construct an OptFormatter object.
 
+        :param conf: The config object from _generator_opts
         :param output_file: a writeable file object
-        :param wrap_width: The maximum length of help lines, 0 to not wrap
         """
         self.output_file = output_file or sys.stdout
-        self.wrap_width = wrap_width
+        self.wrap_width = conf.wrap_width or 998  # arbitrary line length
+        self.minimal = conf.minimal
+        self.summarize = conf.summarize
+
+        if rst2txt:
+            # register the default Sphinx roles
+            for rolename, nodeclass in sphinx_roles.generic_docroles.items():
+                generic = docutils_roles.GenericRole(rolename, nodeclass)
+                docutils_roles.register_local_role(rolename, generic)
+
+            for rolename, func in sphinx_roles.specific_docroles.items():
+                docutils_roles.register_local_role(rolename, func)
+
+            # plus our custom roles
+            for rolename in ('oslo.config:option', 'oslo.config:group'):
+                generic = docutils_roles.GenericRole(rolename,
+                                                     docutils_nodes.strong)
+                docutils_roles.register_local_role(rolename, generic)
 
     def _format_help(self, help_text):
         """Format the help for a group or option to the output file.
 
         :param help_text: The text of the help string
         """
-        if self.wrap_width is not None and self.wrap_width > 0:
+        if rst2txt:
+            help_text = docutils_core.publish_string(
+                source=help_text,
+                writer=rst2txt.Writer(),
+                settings_overrides={'wrap_width': self.wrap_width}
+            ).decode()
+
+            lines = ''
+            for line in help_text.splitlines():
+                lines += '# ' + line + '\n' if line else '#\n'
+
+            lines = [lines]
+        elif self.wrap_width > 0:
             wrapped = ""
             for line in help_text.splitlines():
                 text = "\n".join(textwrap.wrap(line, self.wrap_width,
@@ -192,6 +230,7 @@ class _OptFormatter(object):
             lines = [wrapped]
         else:
             lines = ['# ' + help_text + '\n']
+
         return lines
 
     def _get_choice_text(self, choice):
@@ -199,7 +238,7 @@ class _OptFormatter(object):
             return '<None>'
         elif choice == '':
             return "''"
-        return six.text_type(choice)
+        return str(choice)
 
     def format_group(self, group_or_groupname):
         """Format the description of a group header to the output file
@@ -217,13 +256,11 @@ class _OptFormatter(object):
             lines = ['[%s]\n' % groupname]
         self.writelines(lines)
 
-    def format(self, opt, group_name, minimal=False, summarize=False):
+    def format(self, opt, group_name):
         """Format a description of an option to the output file.
 
         :param opt: a cfg.Opt instance
         :param group_name: name of the group to which the opt is assigned
-        :param minimal: enable option by default, marking it as required
-        :param summarize: output a summarized description of the opt
         :returns: a formatted opt description string
         """
         if not opt.help:
@@ -238,7 +275,7 @@ class _OptFormatter(object):
         if opt.help:
             # an empty line signifies a new paragraph. We only want the
             # summary line
-            if summarize:
+            if self.summarize:
                 _split = opt.help.split('\n\n')
                 opt_help = _split[0].rstrip(':').rstrip('.')
                 if len(_split) > 1:
@@ -247,18 +284,16 @@ class _OptFormatter(object):
             else:
                 opt_help = opt.help
 
-            help_text = u'%s%s (%s)' % (opt_prefix,
-                                        opt_help,
-                                        opt_type)
+            help_text = '%s%s (%s)' % (opt_prefix, opt_help, opt_type)
         else:
-            help_text = u'(%s)' % opt_type
+            help_text = '(%s)' % opt_type
         lines = self._format_help(help_text)
 
         if getattr(opt.type, 'min', None) is not None:
-            lines.append('# Minimum value: %d\n' % opt.type.min)
+            lines.append('# Minimum value: {}\n'.format(opt.type.min))
 
         if getattr(opt.type, 'max', None) is not None:
-            lines.append('# Maximum value: %d\n' % opt.type.max)
+            lines.append('# Maximum value: {}\n'.format(opt.type.max))
 
         if getattr(opt.type, 'choices', None):
             lines.append('# Possible values:\n')
@@ -321,6 +356,14 @@ class _OptFormatter(object):
                 '# effect on stability and/or performance.\n'
             )
 
+        if opt.sample_default:
+            lines.append(
+                '#\n'
+                '# This option has a sample default set, which means that\n'
+                '# its actual default value may vary from the one documented\n'
+                '# below.\n'
+            )
+
         if hasattr(opt.type, 'format_defaults'):
             defaults = opt.type.format_defaults(opt.default,
                                                 opt.sample_default)
@@ -335,7 +378,7 @@ class _OptFormatter(object):
         for default_str in defaults:
             if default_str:
                 default_str = ' ' + default_str.replace('\n', '\n#    ')
-            if minimal:
+            if self.minimal:
                 lines.append('%s =%s\n' % (opt.dest, default_str))
             else:
                 lines.append('#%s =%s\n' % (opt.dest, default_str))
@@ -349,12 +392,12 @@ class _OptFormatter(object):
         """
         self.output_file.write(s)
 
-    def writelines(self, l):
+    def writelines(self, lines):
         """Write an arbitrary sequence of strings to the output file.
 
-        :param l: a list of arbitrary strings
+        :param lines: a list of arbitrary strings
         """
-        self.output_file.writelines(l)
+        self.output_file.writelines(lines)
 
 
 def _cleanup_opts(read_opts):
@@ -520,18 +563,18 @@ def on_load_failure_callback(*args, **kwargs):
     raise
 
 
-def _output_opts(f, group, group_data, minimal=False, summarize=False):
+def _output_opts(f, group, group_data):
     f.format_group(group_data['object'] or group)
     for (namespace, opts) in sorted(group_data['namespaces'],
                                     key=operator.itemgetter(0)):
         f.write('\n#\n# From %s\n#\n' % namespace)
         for opt in sorted(opts, key=operator.attrgetter('advanced')):
             try:
-                if minimal and not opt.required:
+                if f.minimal and not opt.required:
                     pass
                 else:
                     f.write('\n')
-                    f.format(opt, group, minimal, summarize)
+                    f.format(opt, group)
             except Exception as err:
                 f.write('# Warning: Failed to format sample for %s\n' %
                         (opt.dest,))
@@ -676,6 +719,18 @@ def _generate_machine_readable_data(groups, conf):
     return output_data
 
 
+def i18n_representer(dumper, data):
+    """oslo_i18n yaml representer
+
+    Returns a translated to the default locale string for yaml.safe_dump
+
+    :param dumper: a SafeDumper instance passed by yaml.safe_dump
+    :param data: a oslo_i18n._message.Message instance
+    """
+    serializedData = str(data.translation())
+    return dumper.represent_str(serializedData)
+
+
 def _output_machine_readable(groups, output_file, conf):
     """Write a machine readable sample config file
 
@@ -689,6 +744,7 @@ def _output_machine_readable(groups, output_file, conf):
     """
     output_data = _generate_machine_readable_data(groups, conf)
     if conf.format_ == 'yaml':
+        yaml.SafeDumper.add_representer(_message.Message, i18n_representer)
         output_file.write(yaml.safe_dump(output_data,
                                          default_flow_style=False))
     else:
@@ -724,25 +780,27 @@ def generate(conf, output_file=None):
     """
     conf.register_opts(_generator_opts)
 
+    own_file = False
+
     if output_file is None:
-        output_file = (open(conf.output_file, 'w')
-                       if conf.output_file else sys.stdout)
+        if conf.output_file:
+            output_file = open(conf.output_file, 'w')
+            own_file = True
+        else:
+            output_file = sys.stdout
 
     groups = _get_groups(_list_opts(conf.namespace))
 
     if conf.format_ == 'ini':
-        formatter = _OptFormatter(output_file=output_file,
-                                  wrap_width=conf.wrap_width)
+        formatter = _OptFormatter(conf, output_file=output_file)
 
         # Output the "DEFAULT" section as the very first section
-        _output_opts(formatter, 'DEFAULT', groups.pop('DEFAULT'), conf.minimal,
-                     conf.summarize)
+        _output_opts(formatter, 'DEFAULT', groups.pop('DEFAULT'))
 
         # output all other config sections with groups in alphabetical order
         for group, group_data in sorted(groups.items()):
             formatter.write('\n\n')
-            _output_opts(formatter, group, group_data, conf.minimal,
-                         conf.summarize)
+            _output_opts(formatter, group, group_data)
     elif conf.format_ == 'rst':
         _output_human_readable(
             conf.namespace,
@@ -753,10 +811,13 @@ def generate(conf, output_file=None):
                                  output_file=output_file,
                                  conf=conf)
 
+    if own_file:
+        output_file.close()
+
 
 def main(args=None):
     """The main function of oslo-config-generator."""
-    version = pkg_resources.get_distribution('oslo.config').version
+    version = importlib.metadata.version('oslo.config')
     logging.basicConfig(level=logging.WARN)
     conf = cfg.ConfigOpts()
     register_cli_opts(conf)
@@ -765,7 +826,7 @@ def main(args=None):
     except cfg.RequiredOptError:
         conf.print_help()
         if not sys.argv[1:]:
-            raise SystemExit
+            raise SystemExit(1)
         raise
     generate(conf)
 
